#include <bmp.c>
#include <config.c>
#include <bitmapfonts.c>
#include <fonts/captionfont.dat.c>
#include <plot3D.c>
#include <captions.c>
#include <rnd/centremouse.c>
#include <rnd/Max.c>
#include <rnd/noneg.c>
#include <rnd/rnd.c>
#include <rnd/stristr.c>

typedef struct
{
	bmp b[2];
	GLuint t[2]; int txs,tys; float tx1,ty1;
	float ofx,ofy;
	int twoinone;
	Caption caption;
} Slide;

Slide Slide_new(const LSet *pc,const int num=1)
{
	Slide ret;
	char *lfile=configgetnth(pc,"Left image",num),
		*rfile=configgetnth(pc,"Right image",num);
	ret.twoinone=!strcmpi(lfile,rfile);
	ret.b[0]=bmpload(lfile);
	ret.b[1]=bmpload(rfile);
	for (int n=0;n<2;n++) ret.t[n]=bmptexture(ret.b[n],&ret.txs,&ret.tys);
	ret.tx1=(float)ret.b[0].xs/ret.txs;
	ret.ty1=(float)ret.b[0].ys/ret.tys;
	sscanf(configgetnth(pc,"Stereo offset",num),"%f,%f",&ret.ofx,&ret.ofy);
	char *s=configgetnth(pc,"Caption",num);
	ret.caption=Caption_new(s?s:"");
	ret.caption.y=0;
	if (s=stristr(ret.caption.text,"$y=")) {ret.caption.y=atof(s+3)*yres; *s=0;}
	return ret;
}

void Slide_free(Slide s)
{
	for (int n=0;n<2;n++)
	{
		bmpfree(s.b[n]);
		glDeleteTextures(1,&s.t[n]);
	}
	Caption_free(s.caption);
}

WINMAIN
{
	int n; char str[100];
	LSet conf=configload("config.txt");
	int stereomode,stereoswap; plot3D_config(&conf,&stereomode,&stereoswap);
	int slidesleft=configgeti(&conf,"Slides to show"),
		randomise=configgeti(&conf,"Randomise");
	double slidetime=configgetd(&conf,"Slide time"),
		slidepause=configgetd(&conf,"Slide pause before zoom");
	slidetime=Max(slidetime,slidepause+1e-6);
	GRAPHICS(xres,yres,"3D Photo Slideshow [Stephen Brooks 2014-15]");
	loadfontresource(captionfont_dat,"caption");
	srand(GetTickCount());
	int fixedslide=0,num=(fixedslide?fixedslide:randomise?1+rnd(randomise):1);
	Slide s=Slide_new(&conf,num);
	double slidestart=eithertime(),ot=eithertime(),t=0; int paused=0;
	LOOP
	{
		if (KHIT(VK_P)) paused^=1;
		if (KEY(VK_SHIFT))
		{
			if (KHIT(VK_SHIFT)) centremouse(NULL,NULL);
			int dx,dy;
			centremouse(&dx,&dy);
			if (KEY(VK_CONTROL)) s.ofy+=(float)dy/s.b[0].ys; else s.ofx+=(float)dx/s.b[0].xs;
			sprintf(str,"%g,%g",s.ofx,s.ofy); configsetnth(&conf,"Stereo offset",num,str);
		}
		if (!paused) t+=eithertime()-ot; ot=eithertime();
		if (KHIT(VK_LEFT) && num>1) {Slide_free(s); num--; s=Slide_new(&conf,num); slidestart=eithertime(); t=0;}
		if (KHIT(VK_RIGHT) && confighasnth(&conf,"Left image",num+1)) {Slide_free(s); num++; s=Slide_new(&conf,num); slidestart=eithertime(); t=0;}
		if (t>=slidetime)
		{
			Slide_free(s);
			if (slidesleft>0) {slidesleft--; if (slidesleft<=0) exit(0);}
			if (randomise) {int onum=num; num=1+rnd(randomise); if (num==onum) num=1+rnd(randomise);} 
			else if (confighasnth(&conf,"Left image",num+1)) num++; else num=1;
			if (fixedslide) num=fixedslide;
			s=Slide_new(&conf,num);
			slidestart=eithertime(); t=0;
		}
		float zoom=pow(1.75,0.5-0.5*cos(M_PI*noneg((t-slidepause)/(slidetime-slidepause))));
		glEnable(GL_TEXTURE_2D);
		for (n=0;n<2;n++)
		{
			if (stereomode==2 || stereomode==4) glViewport(n*xres/2,0,xres/2,yres);
			else if (stereomode==1 || stereomode==3 || stereomode==5) glViewport(0,n*yres/2,xres,yres/2);
			int oxres=xres,oyres=yres;
			if (stereomode==4) xres/=2; if (stereomode==5) yres/=2;
			float z=zoom*Max((float)xres/s.b[n].xs,(float)yres/s.b[n].ys),
				xs=z*s.b[n].xs,ys=z*s.b[n].ys,
				cx=xres/2+(n-0.5)*s.ofx*xs,cy=yres/2+(n-0.5)*s.ofy*ys,
				htx=0;//0.5/xres; // Not 0.5/s.txs because of mipmapping
			glBindTexture(GL_TEXTURE_2D,s.t[n]);
			if (s.twoinone)
			{
				glBegin(GL_QUADS);
				glTexCoord2f(0.5*n*s.tx1+0.5*htx,0); glVertex2f(cx-0.5*xs,cy+0.5*ys);
				glTexCoord2f(0.5*n*s.tx1+0.5*(s.tx1-htx),0); glVertex2f(cx+0.5*xs,cy+0.5*ys);
				glTexCoord2f(0.5*n*s.tx1+0.5*(s.tx1-htx),s.ty1); glVertex2f(cx+0.5*xs,cy-0.5*ys);
				glTexCoord2f(0.5*n*s.tx1+0.5*htx,s.ty1); glVertex2f(cx-0.5*xs,cy-0.5*ys);
				glEnd();				
			}
			else
			{
				glBegin(GL_QUADS);
				glTexCoord2f(htx,0); glVertex2f(cx-0.5*xs,cy+0.5*ys);
				glTexCoord2f(s.tx1-htx,0); glVertex2f(cx+0.5*xs,cy+0.5*ys);
				glTexCoord2f(s.tx1-htx,s.ty1); glVertex2f(cx+0.5*xs,cy-0.5*ys);
				glTexCoord2f(htx,s.ty1); glVertex2f(cx-0.5*xs,cy-0.5*ys);
				/*glTexCoord2f(htx,0); glVertex2f(cx-0.5*xs,cy+0.5*ys);
				glTexCoord2f(htx,s.ty1); glVertex2f(cx-0.5*xs,cy-0.5*ys);
				glTexCoord2f(htx,s.ty1); glVertex2f(cx-xs,cy-0.5*ys);
				glTexCoord2f(htx,0); glVertex2f(cx-xs,cy+0.5*ys);
				glTexCoord2f(s.tx1-htx,0); glVertex2f(cx+0.5*xs,cy+0.5*ys);
				glTexCoord2f(s.tx1-htx,s.ty1); glVertex2f(cx+0.5*xs,cy-0.5*ys);
				glTexCoord2f(s.tx1-htx,s.ty1); glVertex2f(cx+xs,cy-0.5*ys);
				glTexCoord2f(s.tx1-htx,0); glVertex2f(cx+xs,cy+0.5*ys);*/
				glEnd();
			}
			xres=oxres; yres=oyres;
		}
		glDisable(GL_TEXTURE_2D);
		glViewport(0,0,xres,yres);
		if (stereomode==3) plot3D_interlace(stereoswap);
		Caption_plot(s.caption);
		if (KEY(VK_CONTROL) && KHIT(VK_S)) screenshotauto("3dimage");
		ccl();
	}
	configsave(&conf,"config.txt");
	configfree(&conf);
}
